function [ ] = testacm()

  % INPUT:
  %   The script will ask for:
  %   1. File containing a witness point set W for X.
  %      By default, the script will look for a point file named witness_point_set.
  %      If no such file is found, then the script will look for witness_data in
  %      Bertini's usual format.
  %   2. File containing sample points on a general curve section C of X.
  %      By default, the script will look for a point file named sample_points.
  %   3. Dimension of X.
  % OUTPUT:
  %   Displays several properties, including whether or not X is aCM.
  %   If the coefficients of the Hilbert polynomial cannot be computed using
  %   Matlab's double precision, then a Bertini input file will be generated to
  %   perform this computation.
  
  % Set up constants and start diary
  HFWmaxdeg = 20; % How far to initially compute HF_W
  HFCadddeg = 10; % For nonaCM curves, how many additional terms of HFC to find if needed
  bar = '=========================================================================\n';
  delete('screenout.txt');
  diary('screenout.txt');

  % Ask user for input
  clc;
  fprintf(bar);
  filename1 = input('\nInput W filename or press enter for default (witness_point_set): ','s');
  fprintf('\n');
  if isempty(filename1); filename1 = 'witness_point_set'; end;
  filename2 = input('Input C filename or press enter for default (sample_points): ','s');  
  fprintf('\n');
  if isempty(filename2); filename2 = 'sample_points'; end;
  dimension = input('Input dim(X) or press enter for default (1): ');
  fprintf('\n');
  if isempty(dimension); dimension = 1; end;
    
  % Find HF_W, index_W, and HF_C
  fprintf(bar);
  [HFW,numcoords] = ComputeHF(filename1,HFWmaxdeg,1);
  if HFW == -1;
    diary off; return;
  end;
  index_W = -2;
  for i=1:HFWmaxdeg-1
    if HFW(i) == HFW(i+1) && index_W == -2
      index_W = i-1;
    end;
  end;
  if index_W == -2
    fprintf(['Could not find index(W).\n\n',bar]);
    return;
  end;
  degree = HFW(index_W+1);
  [HFC,numcoords] = ComputeHF(filename2,index_W+1,0);
  if HFC == -1;
    diary off; return;
  end;

  % Display Hilbert functions for W and C
  fprintf(['\n',bar]);
  fprintf(['\nHF_C =']);
  disp(HFC');
  fprintf('diff =');
  DeltaHFC = [1;diff(HFC)];
  disp(DeltaHFC');
  fprintf(['HF_W =']);
  disp(HFW(1:size(HFC,1),1)');

  % Find regularities for W and C
  index_C = -2;
  for i=1:size(DeltaHFC,1)-1%index_W+1
    if DeltaHFC(i) == DeltaHFC(i+1) && index_C == -2
      index_C = i-2;
    end;
  end;
  if index_C == -2
    fprintf('Result: NOT aCM\n');
    fprintf('Could not find index(C) based on the above HF.\n');
    yesorno = input('Compute more terms of HF_C? (yes): ','s');
    fprintf('\n');
    if isempty(yesorno); yesorno = 'yes'; end;
    if strcmpi(yesorno(1), 'n')
      fprintf(bar);
      diary off; return;      
    end;
    % Compute more terms of HF_C, looking for reg_C
    fprintf('Computing more terms of HF_C...\n');
    [HFC,numcoords] = ComputeHF(filename2,index_W+1+HFCadddeg,index_W);
    % Display new results
    fprintf(['\n',bar]);
    fprintf(['\nHF_C =']);
    disp(HFC(1:nnz(HFC))');  
    fprintf('diff =');
    DeltaHFC = [1;diff(HFC)];
    disp(DeltaHFC(1:nnz(HFC))');
    fprintf(['HF_W =']);
    disp(HFW(1:nnz(HFC),1)');    
    for i=1:size(DeltaHFC,1)-1
      if DeltaHFC(i) == DeltaHFC(i+1) && index_C == -2
        index_C = i-2;
      end;
    end;
    if index_C == -2
      fprintf('Result: NOT aCM\n');
      fprintf(['Could not find index(C).\n\n',bar]);   
      diary off; return;
    end;     
  end;
  reg_C = max(index_W+1,index_C+1);
  fprintf(['index(W) = ',num2str(index_W),'\n']);
  fprintf(['index(C) = ',num2str(index_C),'\n']);
  fprintf(['reg(W) = ', num2str(index_W+1),'\n']);
  fprintf(['reg(C) = ', num2str(reg_C),'\n']);

  % Determine result of aCM test
  % (Note: 1:index_W+2 corresponds to 0\leq t\leq indeX_W+1)
  if HFW(1:index_W+2,1)==DeltaHFC(1:index_W+2,1)
    fprintf('Result: YES aCM\n');
    acm = 1; 
  else
    fprintf('Result: NOT aCM\n');
    acm = 0; 
  end;

  % Arithmetic genus of curve
  fprintf(['degree = ',num2str(degree),'\n']);
  if index_C+1 < 1
    arithgenus = 1 + (index_C)*degree;
  else
    arithgenus = 1 - HFC(index_C+1) + (index_C)*degree;
  end;
  fprintf(['arithgenus(C) = ',num2str(arithgenus),'\n']);

  % Hilbert polynomial of the curve
  fprintf(['HP_C(t) = ',num2str(degree), '*t-',num2str(arithgenus-1),'\n']);

  % Hilbert series of the curve
  hpoly = '';
  hvector = [1;diff(DeltaHFC)];
  for i=1:index_C+2
    if hvector(i,1)~=1 || i==1
   hpoly = [hpoly,num2str(hvector(i,1))];
    end;
    if i > 1
   if hvector(i,1)~=1
     hpoly = [hpoly,'*'];
   end;
   hpoly = [hpoly,'t'];
   if i > 2
     hpoly = [hpoly,'^',num2str(i-1)];
   end;
    end;
    if i < index_C+2
   hpoly = [hpoly,'+'];
    end;
  end;
  fprintf(['HS_C(t) = (',hpoly,')/(1-t)^2\n\n']);

  % If we're just dealing with a curve, then we're finished.
  if dimension==1
    fprintf([bar]);
    diary off; return;
  end;

  % If higher dimensional but not aCM, then we're finished.
  if acm==0
    fprintf('The following were not computed, because X is non aCM:\n\n');
    fprintf(['reg(X) = ?\nindex(X) = ?\nHS_X(t) = ?\nHF_X(t) = ?\nHP_X(t) = ?\n\n',bar]);
    diary off; return;
  end;

  % Regularities for X
  fprintf(['reg(X) = ',num2str(reg_C),'\n']);
  index_X = reg_C-dimension-1;
  fprintf(['index(X) = ',num2str(index_X),'\n']);

  % Hilbert series of X
  fprintf(['HS_X(t) = (',hpoly,')/(1-t)^',num2str(dimension+1),'\n\n']);

  % Append more entries to HFC which will be needed soon for HP_X
  for i=1:dimension+1
    HFC = [HFC;HFC(size(HFC,1),1)+degree];
  end;

  % Hilbert function of X
  HFtemp = HFC;
  HFX = zeros(size(HFC,1),1);
  for i=1:dimension-1
    HFX(1) = 1;
    for j=2:size(HFtemp,1)
      HFX(j) = HFX(j-1) + HFtemp(j);
    end;
    HFtemp = HFX;
  end;   
  fprintf(['HF_X =']);
  disp(HFX(1:index_W+2,1)');

  % Hilbert polynomial of X
  HPX = ComputeHP(dimension, index_X, HFX);
  if HPX~=0
    fprintf('\nHP_X(t) = ');
    for i = 1:size(HPX,1)
      c = HPX(i,1)/HPX(i,2);
      if isequal(fix(c), c)
        if c~=1 || i==1
          fprintf(num2str(c));
        end;
      else
        fprintf([num2str(HPX(i,1)),'/',num2str(HPX(i,2))]);
      end;
      if i>1
        if c~=1
          fprintf('*');
        end;
        fprintf('t');
        if i>2
          fprintf(['^',num2str(i-1)]);
        end;
      end;
      if i<size(HPX,1)
        fprintf('+');
      end;
    end;
  end;
  
  fprintf(['\n\n',bar]);
  diary off;
  
return;


function [HFResult,numcoordsf] = ComputeHF(filenamepoints,HFMaxDeg,stopcondition)

  % Make sure file exists and decide if conversion is needed
  convert = 0;
  fprintf(['\nUsing file: ',filenamepoints,'\n']);
  if ~exist(filenamepoints,'file')
    fprintf('File not found!\n\n');
    if strcmp(filenamepoints,'witness_point_set')
      fprintf('Looking for witness_data instead... ');
      if ~exist('witness_data','file')
        fprintf('not found!\n\n');
        HFResult = -1;
        return;        
      else
        fprintf('ok.\n');
        convert = 1;
      end;      
    else
      HFResult = -1;
      return;
    end;
  else
    if strcmp(filenamepoints,'witness_data')
      convert = 1;
      filenamepoints = 'witness_point_set';
      fprintf('witness_data found.\n');
    end;      
  end;

  % Convert witness_data to witness_point_set
  if convert==1
    fprintf('Creating witness_point_set from witness_data... ');
    % Take care of the first few lines of the file
    fileIn = fopen('witness_data','r');
    numcoords = str2num(fgetl(fileIn));        
    textline = fgetl(fileIn);
    textline = fgetl(fileIn);
    numpoints = str2num(fgetl(fileIn));
    textline = fgetl(fileIn);
    fileOut = fopen('witness_point_set','w');
    fprintf(fileOut,[num2str(numpoints),'\n\n']);
    % Write the points
    for i=1:numpoints
      for c=1:numcoords
        textline = fgetl(fileIn);
        fprintf(fileOut,[textline,'\n']);
      end;
      fprintf(fileOut,'\n');
      for c=1:numcoords+10
        textline = fgetl(fileIn);
      end;          
    end;
    fprintf(fileOut,'\n');
    fclose(fileIn);
    fclose(fileOut);
    fprintf('ok.\n');
  end;  

  % Determine number of coordinates
  fileIn = fopen(filenamepoints,'r');
  textline = fgetl(fileIn);
  fprintf(['Found ',textline,' points in ']);
  textline = fgetl(fileIn);
  numcoordsf = -1;
  while(strcmp(textline,'')==0 || numcoordsf==-1)
    textline = fgetl(fileIn);
    numcoordsf = numcoordsf + 1;
  end;
  fclose(fileIn);
  fprintf([num2str(numcoordsf), ' coordinates.\n\n']);

  % Find Hilbert function
  [n,Y] = ReadInStart(numcoordsf, filenamepoints);
  Y1 = [ones(size(Y,1),1),Y];
  HFResult = HilbertPoints(Y1,1e-8,HFMaxDeg,stopcondition);

return;


function [H] = ComputeHP(dim,r,HF)

  fprintf('Computing HP_X... ');

  k=dim;
  
  % Create matrix A
  A = zeros(k+1,dim+1);
  for j=0:k
    for e=0:dim
      A(j+1,e+1)=((r+j)^e);
    end;
  end;
  
  % Create column vector b
  b = zeros(k+1,1);
  for j = 0:k
    if r+j+1 < 1
      b(j+1) = 0;
    else
      b(j+1) = HF(r+j+1);
    end;
  end;
  
  % Solve for column vector x
  x = A\b;
  
  % Obtain rational numbers for the HP coefficients
  for i=1:size(x)
    [C(i,1) C(i,2)] = rat(x(i));
  end; 
  H = C; 
  
  % Check the HP
  fprintf('checking... ');
  row = zeros(1,size(H,1));
  for i=1:size(H,1)
    row(1,i)=H(size(H,1)-i+1,1)/H(size(H,1)-i+1,2);
  end;
  fun=@(t) poly2sym(row,t);
  error = 0;
  for t=r:r+dim+1
    %fprintf(['  Checking HP(',num2str(t),')=HF(',num2str(t),')... ']);   
    evaluate = double(feval(fun,t));
    %fprintf([num2str(evaluate,16),' = ',num2str(HF(t+1)),'?  ']);
    if t+1 > 0
      check = HF(t+1);
    else
      check = 0;
    end;
    if evaluate==check
      %fprintf('ok.\n');
    else
      %fprintf('ERROR\n');
      error = 1;
      %break;
    end;
  end;    
 
  % If the HP failed, then create a Bertini input file
  if error==0
    fprintf('ok.\n');
  else
    H = 0;
    fprintf('double precision failed.\nCreating Bertini input file... ');
    fileBertini = fopen('input.hp','w');
    fprintf(fileBertini,'CONFIG\n\nMPTYPE: 1;\nPRECISION: 512;\nSINGVALZEROTOL: 1e-50;\nSHARPENDIGITS: 100;\n\nEND;\n\nINPUT\n\nvariable_group ');
    for j = 1:k+1
      fprintf(fileBertini,['x',num2str(j)]);
      if j<k+1
        fprintf(fileBertini,', ');
      end;
    end;
    fprintf(fileBertini,';\nfunction ');
    for j = 1:k+1
      fprintf(fileBertini,['f',num2str(j)]);
      if j<k+1
        fprintf(fileBertini,', ');
      end;
    end;    
    fprintf(fileBertini,';\n\n');
    for j = 1:k+1
      c = ['f',num2str(j),' = '];
      for l = 1:dim+1
        c = [c,'(',num2str(r+j-1),')^',num2str(l-1),'*x',num2str(l),' + '];
      end;
      c = [c,'-',num2str(b(j)),';'];
      fprintf(fileBertini,c);
      fprintf(fileBertini,'\n');
    end;
    fprintf(fileBertini,'\nEND;');
    fclose(fileBertini);
    fprintf('ok.\n');
  end;
  
return;


function [HF, SV] = HilbertPoints(Points,tol,maxDeg,stopcondition)  

  % explanation of stopcondition argument:
  % if =0, ignored; go up to maxDeg
  % if =1, stop if two subsequent values are equal,
  %        or if maxDeg is reached (use for HF_W)
  % if >1, stop if (two subsequent values of \Delta HF
  %        are equal AND degree>stopcondition) or if
  %        maxDeg is reached (use when computing additional
  %        terms of HF_C for a nonaCM curve)

  % find the number of Points
  numPoints = size(Points, 1);

  % allocate HF
  HF = zeros(maxDeg+1,1);

  % compute intrinsic coordinates
  V = VeroneseMatrix(Points, 1);
  N = null(V);
  if size(N,2) == 0
    % full rank
    Int = Points;
  else
    % work intrinsically
    N = null(transpose(N));
    Int = transpose(N \ transpose(Points));
  end;
    
  % compute the Hilbert function using the intrinsic coordinates
  HF(1) = 1;
  SV = struct('maxSV',1,'minNonzeroSV',1,'maxZeroSV',0);
  disp('HF[0] = 1');  
  for degree = 1:maxDeg
    if stopcondition>1 && degree>stopcondition
      A = [1;diff(HF)];
      if A(degree-1)==A(degree)
        return;
      end;
    end;
    if stopcondition==1 && degree > 1
      if HF(degree) == HF(degree-1)
        return;
      end;
    end;          
    if degree > 1 && HF(degree) == numPoints
      HF(degree+1) = numPoints;
      disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
    elseif degree > 1 && HF(degree) == HF(degree-1)
      HF(degree+1) = HF(degree);
      disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
    else
      %disp('setup VM');
      V = VeroneseMatrix(Int, degree);
      %disp('SVD');
      S = svd(V);
      HF(degree+1) = sum(S > tol);
      SV.maxSV(degree+1) = S(1);
      SV.minNonzeroSV(degree+1) = S(HF(degree+1));
      if HF(degree+1) < size(S,1);
        SV.maxZeroSV(degree+1) = S(HF(degree+1)+1);
      else
        SV.maxZeroSV(degree+1) = 0;
      end;
      disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1)),'  (max: ',num2str(SV.maxSV(degree+1)),' min: ',num2str(SV.minNonzeroSV(degree+1)),' zero: ',num2str(SV.maxZeroSV(degree+1)),')']);      
    end;
  end;

return;


function [numPts, Pts] = ReadInStart(numVars, fileLocation)

  % open file
  RF = fopen(fileLocation, 'r');

  % make sure the file exists
  if RF == -1
    % file does not exist
    disp(['The file ''', fileLocation, ''' does not exist!']);
    numPts = 0;
    Pts = 0;
    return;
  end;

  % read in the number of points
  numPts = fscanf(RF, '%d', 1);

  % setup Pts
  Pts = zeros(numPts, numVars);

  % loop through to read in the points
  tempR = 0; tempI = 0;
  for k = 1:numPts
    A = fscanf(RF,'%e',2*numVars);
    Pts(k,:) = A(1:2:end) + 1i*A(2:2:end);
  end;

  % close file
  fclose(RF);

return;                  


function [A] = setupDegree(deg, n)
  % setup degrees for Veronese embedding

  numRows = nchoosek(n+deg-1,deg);
  A = zeros(numRows,n);
  A(1,1) = deg;

  for j = 1:numRows-1
    % copy
    A(j+1,:) = A(j,:);
    % find target entry
    target = -1;
    for k = n-1:-1:1
      if A(j+1,k) ~= 0
        target = k;
        break;
      end;
    end;
    % move entry
    A(j+1,target) = A(j+1,target) - 1;
    A(j+1,target+1) = A(j+1,target+1) + 1;
    % move all of final entry to target+1 if not the same!
    if target ~= n-1
      A(j+1,target+1) = A(j+1,target+1) + A(j+1,n);
      A(j+1,n) = 0;
    end;
  end;
return;


function [M] = veronese(X, A)
  % compute the Veronese embedding based on A
  N = size(A,1);
  M = zeros(1,N);
  for j = 1:N
    M(1,j) = prod(X.^A(j,:));
  end;
return;


function [VM] = VeroneseMatrix(Pts, deg, normalize)
  % compute the Veronese embedding of 'Pts' in degree 'deg'

  % default
  if nargin < 3
    normalize = 1;
  end;

  N = size(Pts,1); % number of points
  numVars = size(Pts,2); % number of variables

  % initialize VM
  VM = zeros(N,nchoosek(numVars + deg - 1, deg));
  %disp(['size: ',num2str(size(VM,1)),' x ',num2str(size(VM,2))]);

  % setup A
  %disp('setup degree');
  A = setupDegree(deg, numVars);

  % loop to setup VM
  for j = 1:N
    %if mod(j,10000) == 1 disp([num2str(j),' of ',num2str(N)]); end;
    % normalize Pts
    if normalize == 0
      P = Pts(j,:);
    else
      P = Pts(j,:) / norm(Pts(j,:));
    end;
    
    % compute embedding
    if normalize == 0
      VM(j,:) = veronese(P,A);
    else
      v = veronese(P,A);
      VM(j,:) = v/norm(v);
    end;
  end;
return;
